@@ -1,6 +1,9 @@
 package VCS::CMSynergy::Client;
 
-our $VERSION = do { (my $v = q$Revision: 335 $) =~ s/^.*:\s*//; $v };
+# Copyright (c) 2001-2010 argumentum GmbH, 
+# See COPYRIGHT section in VCS/CMSynergy.pod for usage and distribution rights.
+
+our $VERSION = do { (my $v = q$Revision: 381 $) =~ s/^.*:\s*//; $v };
 
 =head1 NAME
 
@@ -432,7 +435,7 @@ __PACKAGE__->_memoize_method(_version => sub
     return $self->set_error($err || $out) unless $rc == 0;
 
     my %version;
-    my $cmsynergy_rx = qr{(?:Continuus|CM Synergy|SYNERGY/CM|Telelogic Synergy)};
+    my $cmsynergy_rx = qr{(?:CM Synergy|SYNERGY/CM|Telelogic Synergy|IBM Rational Synergy)};
     ($version{cmsynergy}) = $out =~ /^$cmsynergy_rx Version\s+(\S*)$/imo
 	or return $self->set_error("can't recognize version from `$out'");
     ($version{short}) = $version{cmsynergy} =~ /^(\d+\.\d+)/;
@@ -453,7 +456,7 @@ sub ps
     # "ps" is not a recognized "interactive" command
     local $this->{coprocess} = undef;
 
-    my ($rc, $out, $err) = $this->_ccm(ps => @filter);
+    my ($rc, $out, $err) = $this->_ccm(qw/ps/, @filter);
     return $this->set_error($err || $out) unless $rc == 0;
 
     # split at "rfc address..." header lines;
@@ -1103,13 +1106,5 @@ L<VCS::CMSynergy::Object>
 
 Roderich Schupp, argumentum GmbH <schupp@argumentum.de>
 
-=head1 COPYRIGHT AND LICENSE
-
-The VCS::CMSynergy::Client module is Copyright (c) 2001-2005 argumentum GmbH, 
-L<http://www.argumentum.de>.  All rights reserved.
-
-You may distribute it under the terms of either the GNU General Public
-License or the Artistic License, as specified in the Perl README file.
-
 =cut
 
