@@ -29,10 +29,10 @@ sub update
     my $current_users;
     {
 	local $/ = undef;		# slurp in whole file
-	open(CURRENT_USERS, "<$current_users_file")
+	open my $fh, "<", $current_users_file
 	    or die "can't open `$current_users_file': $!";
-	$current_users = <CURRENT_USERS>;
-	close(CURRENT_USERS);
+	$current_users = <$fh>;
+	close $fh;
     }
     chomp($current_users);		# because VCS::CMSynergy also did chomp
 
